local Addon = _G[...]
local LibKeyBound = LibStub("LibKeyBound-1.0")
local ShowOverlayGlow = LibStub("LibButtonGlow-1.0").ShowOverlayGlow
local HideOverlayGlow = LibStub("LibButtonGlow-1.0").HideOverlayGlow

local DEFAULT_STATE = "default"

local SLOT_COLORS = {
    normal = {1, 1, 1},
    oor = {1, 0.3, 0.1},
    oom = {0.1, 0.3, 1},
    unusable = {0.4, 0.4, 0.4}
}

local BasicSlot = Addon.ActionSlots:CreateKind("basic")

function BasicSlot:OnLoad()
    self.flashing = 0
	self.flashtime = 0
	self.action = 9999

	self:SetAttribute("kind", "empty")
	self:SetAttribute("checkselfcast", true)
	self:SetAttribute("checkfocuscast", true)
	self:RegisterForDrag("LeftButton", "RightButton")
	self:RegisterForClicks("AnyUp")
	self.Border:SetVertexColor(0, 1, 0, 0.35)
	self.NormalTexture:SetAlpha(0.5)

	if not self.FloatingBG then
		self.FloatingBG = self:CreateTexture(nil, "BACKGROUND", nil, -1)
		self.FloatingBG:SetTexture([[Interface\Buttons\UI-Quickslot]])
		self.FloatingBG:SetPoint("TOPLEFT", -15, 15)
		self.FloatingBG:SetPoint("BOTTOMRIGHT", 15, -15)
		self.FloatingBG:SetAlpha(0.4)
	end

	self:Execute(([[ DEFAULT_STATE = %q ]]):format(DEFAULT_STATE))

	self:SetAttribute("lockactionbar", tonumber(LOCK_ACTIONBAR) == 1)

	-- usage: SetAs("key1", value1, ..., keyN, valueN)
	self:SetAttribute("SetAttributes", [[
		local changed = false

		for i = 1, select("#", ...), 2 do
			local key, value = select(i, ...)
			if self:GetAttribute(key) ~= value then
				self:SetAttribute(key, value)
				changed = true
			end
		end

		return changed
	]])

	-- usage: changed = SetKind(kind, action)
	self:SetAttribute("SetKind", [[
		-- SetKind
		local kind, action = ...

		local state = self:GetAttribute("state") or DEFAULT_STATE
		local kindKey = strjoin("--", "kind", state)
		local actionKey = strjoin("--", "kaction", state)

		kind = kind or "empty"
		action = action or ""

		if self:RunAttribute("SetAttributes", kindKey, kind, actionKey, action) then
			self:RunAttribute("UpdateKind")
			return true
		end
	]])

	-- usage: kind, action = GetKind()
	self:SetAttribute("GetKind", [[
		local state = self:GetAttribute("state") or DEFAULT_STATE
		local kindKey = strjoin("--", "kind", state)
		local actionkey = strjoin("--", "kaction", state)
		local kind = self:GetAttribute(kindKey)
		local action = self:GetAttribute(actionkey) or ""

		if kind then
			return kind, action
		end

		kindKey = strjoin("--", "kind", DEFAULT_STATE)
		actionkey = strjoin("--", "kaction", DEFAULT_STATE)
		kind = self:GetAttribute(kindKey) or "empty"
		action = self:GetAttribute(actionkey) or ""

		return kind, action
	]])

	self:SetAttribute("UpdateKind", [[
		local forceUpdate = ...
		local kind, action = self:RunAttribute("GetKind")

		if self:RunAttribute("SetAttributes", "kind", kind, "kaction", action) or forceUpdate then
			self:RunAttribute("OnKindUpdated", kind, action, forceUpdate)
			self:RunAttribute("UpdateBindings")
		end
	]])

	self:SetAttribute("UpdateBindings", [[
		self:GetFrameRef("BindingsManager"):RunAttribute("UpdateFrameBindings", self:GetName())
		self:CallMethod("UpdateHotkey")
	]])

	self:SetAttribute("OnStateUpdated", [[
		self:RunAttribute("UpdateKind")
	]])

	self:SetAttribute("OnKindUpdated", [[
		-- OnKindUpdated
		local kind, action, forceUpdate = ...
		local handler = strjoin("-", "OnKindUpdated", kind)

		if (self:GetAttribute(handler) and self:RunAttribute(handler, action)) or forceUpdate then
			self:CallMethod("OnKindUpdated", kind, action)
		end
	]])

	self:SetAttribute("OnKindUpdated-actionpage", [[
		local id = ...

		return self:RunAttribute("SetAttributes",
			"type", "actionbar",
			"action", id,
			"bindaction", ""
		)
	]])

	self:SetAttribute("OnKindUpdated-basic", [[
		return self:RunAttribute("SetAttributes", "type", "empty")
	]])

	self:SetAttribute("OnKindUpdated-battlepet", ([[
		local battlePetID = ...
		local SUMMON_BATTLE_PET_COMMAND = %q
		local SUMMON_RANDOM_BATTLE_PET_COMMAND = %q
		local macrotext

		if battlePetID == "BattlePet-0-FFFFFFFFFFFFFF" then
			macrotext = SUMMON_RANDOM_BATTLE_PET_COMMAND
		else
			macrotext = strjoin(" ", SUMMON_BATTLE_PET_COMMAND, battlePetID)
		end

		return self:RunAttribute("SetAttributes",
			"type", "macro",
			"macro", nil,
			"macrotext", macrotext,
			"macrotext2", "",
			"battlepet", battlePetID,
			"bindaction", ""
		)
	]]):format(SLASH_SUMMON_BATTLE_PET1, SLASH_RANDOMFAVORITEPET1))

	self:SetAttribute("OnKindUpdated-equipmentset", ([[
		local setName = ...
		local macrotext = strjoin(" ", %q, setName)

		return self:RunAttribute("SetAttributes",
			"type", "macro",
			"macro", nil,
			"macrotext", macrotext,
			"macrotext2", "",
			"equipmentset", setName,
			"bindaction", ""
		)
	]]):format(SLASH_EQUIP_SET1))

	self:SetAttribute("OnKindUpdated-empty", [[
		return self:RunAttribute("SetAttributes",
			"type", "empty",
			"bindaction", ""
		)
	]])

	self:SetAttribute("OnKindUpdated-exitvehicle", [[
		return self:RunAttribute("SetAttributes",
			"type", "leavevehicle",
			"bindaction", ""
		)
	]])

	self:SetAttribute("OnKindUpdated-flyout", [[
		local flyoutID = ...

		return self:RunAttribute("SetAttributes",
			"type", "flyout",
			"spell", flyoutID,
			"bindaction", ""
		)
	]])

	self:SetAttribute("OnKindUpdated-item", [[
		local itemString = ...

		return self:RunAttribute("SetAttributes",
			"type", "item",
			"item", itemString,
			"bindaction", ""
		)
	]])

	self:SetAttribute("OnKindUpdated-macro", [[
		local macroID = ...

		return self:RunAttribute("SetAttributes",
			"type", "macro",
			"macro", macroID,
			"macrotext", "",
			"macrotext2", "",
			"bindaction", ""
		)
	]])

	self:SetAttribute("OnKindUpdated-mount", [[
		local mountID = ...
		local macrotext = ("/run C_MountJournal.SummonByID(%d)"):format(mountID)

		return self:RunAttribute("SetAttributes",
			"type", "macro",
			"macro", "",
			"macrotext", macrotext,
			"mount", mountID,
			"bindaction", ""
		)
	]])

	self:SetAttribute("OnKindUpdated-movement", [[
		local action = ...

		return self:RunAttribute("SetAttributes",
			"type", "movement",
			"movement", action,
			"bindaction", action
		)
	]])

	self:SetAttribute("OnKindUpdated-overrideaction", [[
		local action = tonumber(...)
		local offset = (self:GetAttribute("overrideIndex") or 0)

		return self:RunAttribute("SetAttributes",
			"type", "action",
			"action", action + offset,
			"bindaction", ""
		)
	]])

	self:SetAttribute("OnKindUpdated-petability", [[
		local args = ...
		local petActionID, name, subtext = strsplit("|", args)
		local macrotext = ("/cast %s(%s)"):format(name, subtext)
		local macrotext2 = ("/petautocasttoggle %s"):format(name)

		return self:RunAttribute("SetAttributes",
			"type", "macro",
			"macro", "",
			"macrotext", macrotext,
			"macrotext2", macrotext2,
			"petActionID", tonumber(petActionID),
			"bindaction", ""
		)
	]])

	self:SetAttribute("OnKindUpdated-petcommand", [[
		local args = ...
		local petActionID, macrotext = strsplit("|", args)

		return self:RunAttribute("SetAttributes",
			"type", "macro",
			"macro", "",
			"macrotext", macrotext,
			"macrotext2", "",
			"petActionID", tonumber(petActionID),
			"bindaction", ""
		)
	]])

	self:SetAttribute("OnKindUpdated-spell", [[
		local spellID = ...

		return self:RunAttribute("SetAttributes",
			"type", "spell",
			"spell", spellID,
			"bindaction", ""
		)
	]])

	self:SetAttribute("_childupdate-currentState", [[
		local state = message or DEFAULT_STATE

		if self:RunAttribute("SetAttributes", "state", state) then
			self:RunAttribute("OnStateUpdated", state)
		end
	]])

	self:SetAttribute("_childupdate-overrideIndex", [[
		local overrideIndex = message or 0

		if self:RunAttribute("SetAttributes", "overrideIndex", overrideIndex) then
			self:RunAttribute("UpdateKind", true)
		end
	]])

	self:SetScript("PostClick", self.OnPostClick)
	self:SetScript("OnEnter", self.OnEnter)
	self:SetScript("OnLeave", self.OnLeave)

	Addon:Notify("OnActionSlotLoaded", self)
end

function BasicSlot:OnActivate()
	self:Show()
	Addon:Notify("OnActionSlotActivated", self)
end

function BasicSlot:OnDeactivate()
	self:Hide()
	Addon:Notify("OnActionSlotDeactivated", self)
end

function BasicSlot:OnDrop(kind, ...)
	if InCombatLockdown() then return end

	local pKind, pAction = self:GetKind()

	if self:PlaceAction(kind, ...) and pKind then
		Addon.SecureEnv:PickupAction(pKind, pAction)
	end
end

function BasicSlot:OnEnter()
	LibKeyBound:Set(self)

	if not self:HasAction() then return end

	local show = Addon:ShowingActionButtonTooltips() and (
		(not InCombatLockdown())
		or Addon:ShowingActionButtonTooltipsInCombat()
		or LibKeyBound:IsShown()
	)

	if show then
		local tooltip = _G.GameTooltip

		if GetCVar("UberTooltips") == "1" then
			GameTooltip_SetDefaultAnchor(tooltip, self)
		else
			tooltip:SetOwner(self, "ANCHOR_RIGHT")
		end

		self:SetTooltip(tooltip)
	end
end

function BasicSlot:OnLeave()
	local tooltip = _G.GameTooltip

	if tooltip:IsOwned(self) then
		tooltip:Hide()
	end
end

function BasicSlot:OnKindUpdated(kind, action)
	local state = self:GetAttribute("state") or DEFAULT_STATE

	local config = self.sets[state]
	if config == nil then
		config = { kind = kind, kaction = action }
		self.sets[state] = config
	else
		config.kind = kind
		config.kaction = action
	end

	Addon.ActionSlots:SetKind(self, kind)
	self:Update()
end

function BasicSlot:OnPostClick()
	self:UpdateState()
end

function BasicSlot:Activate()
	if not self:GetAttribute("active") then
		self:SetAttribute("active", true)
		self:OnActivate()
	end
end

function BasicSlot:Deactivate()
	if self:GetAttribute("active") then
		self:SetAttribute("active", false)
		self:OnDeactivate()
	end
end

local function basicSlot_SetKind(self, stateID, kind, action)
	return self:SetAttributes(
		strjoin("--", "kind", stateID), kind,
		strjoin("--", "kaction", stateID), action
	)
end

function BasicSlot:Initialize(settings)
	if InCombatLockdown() then return end

	if type(settings) ~= "table" then
		error("Usage BasicSlot:Initialize{settings}", 2)
		return
	end

	local oldSets = self.sets
	local newSets = settings
	self.sets = settings

	-- clear any setting not explicitly set in new settings
	if oldSets then
		for stateID in pairs(oldSets) do
			if not rawget(newSets, stateID) then
				basicSlot_SetKind(self, stateID, nil, nil)
			end
		end
	end

	-- set explicit settings from new settings
	for stateID, contents in pairs(newSets) do
		basicSlot_SetKind(self, stateID, contents.kind, contents.kaction)
	end

	-- apply new defaults
	local default = newSets.default
	basicSlot_SetKind(self, DEFAULT_STATE, default.kind, default.kaction)

	self:Execute([[ self:RunAttribute("UpdateKind") ]])
	self:Execute([[ self:RunAttribute("UpdateBindings") ]])
end

function BasicSlot:PlaceAction(kind, ...)
	if kind == "spell" then
		local spellID = select(3, ...)
		return self:SetKind("spell", spellID)
	elseif kind == "actionpage" then
		local pageID = ...
		return self:SetKind("actionpage", pageID)
	elseif kind == "item" then
		local itemString = select(2, ...)
		return self:SetKind("item", itemString)
	elseif kind == "macro" then
		return self:SetKind("macro", ...)
	elseif kind == "flyout" then
		return self:SetKind("flyout", ...)
	elseif kind == "equipmentset" then
		return self:SetKind("equipmentset", ...)
	elseif kind == "exitvehicle" then
		return self:SetKind("exitvehicle", ...)
	elseif kind == "mount" then
		return self:SetKind("mount", ...)
	elseif kind == "movement" then
		return self:SetKind("movement", ...)
	elseif kind == "petcommand" then
		local id, command = ...
		return self:SetKind("petcommand", strjoin("|", id, command))
	elseif kind == "petability" then
		local id, name, subname = ...
		return self:SetKind("petability", strjoin("|", id, name, subname))
	elseif kind == "battlepet" then
		return self:SetKind("battlepet", ...)
	elseif kind == "overrideaction" then
		return self:SetKind("overrideaction", ...)
	elseif (not kind or kind == "empty") then
		return self:SetKind("empty")
	end
end

local function flashingAnimation_Create(slot)
	local animation = slot.Flash:CreateAnimationGroup()

	animation:SetLooping("BOUNCE")

	local alpha = animation:CreateAnimation("ALPHA")

	alpha:SetDuration(ATTACK_BUTTON_FLASH_TIME)
	alpha:SetFromAlpha(0)
	alpha:SetToAlpha(1)

	return animation
end

function BasicSlot:StartFlashing()
	self.Flash:Show()
	self.flashAnimation = self.flashAnimation or flashingAnimation_Create(self)
	self.flashAnimation:Play()
end

function BasicSlot:StopFlashing()
	if self.flashAnimation then
		self.Flash:Hide()
		self.flashAnimation:Stop()
	end
end

function BasicSlot:Update()
	self:UpdateHasAction()
	self:UpdateIcon()

	if self:HasAction() then
		if not self:GetAttribute("statehidden") then
			self:Show()
		end

		self:UpdateState()
		self:UpdateCooldown()
		self:UpdateFlash()
		self:UpdateHighlightMark()
		self:UpdateSpellHighlightMark()
	else
		self.cooldown:Hide()
		ClearChargeCooldown(self)

		self:SetChecked(false)
	end

	self:UpdateGrid()
	self:UpdateEquipped()
	self:UpdateText()

	self:UpdateCount()

	self:UpdateFlyout()
	self:UpdateOverlayGlow()

	self:UpdateTooltip()

	self:UpdateAutoCast()
end

function BasicSlot:UpdateAutoCast()
	local autocast, enabled = self:IsAutoCastAction()
	local show = autocast and enabled

	self.AutoCastable:SetShown(show)

	if show then
		self.AutoCastShine:Show()
		AutoCastShine_AutoCastStart(self.AutoCastShine)
	else
		self.AutoCastShine:Hide()
		AutoCastShine_AutoCastStop(self.AutoCastShine)
	end
end

function BasicSlot:UpdateCooldown()
	local locStart, locDuration = self:GetActionLossOfControlCooldown()
	local start, duration, enable, modRate = self:GetActionCooldown()
	local charges, maxCharges, chargeStart, chargeDuration, chargeModRate = self:GetActionCharges()

	if (locStart + locDuration) > (start + duration) then
		if self.cooldown.currentCooldownType ~= COOLDOWN_TYPE_LOSS_OF_CONTROL then
			self.cooldown:SetEdgeTexture("Interface\\Cooldown\\edge-LoC");
			self.cooldown:SetSwipeColor(0.17, 0, 0);
			self.cooldown:SetHideCountdownNumbers(true);
			self.cooldown.currentCooldownType = COOLDOWN_TYPE_LOSS_OF_CONTROL;
		end

		CooldownFrame_Set(self.cooldown, locStart, locDuration, true, true, modRate)
		ClearChargeCooldown(self)
	else
		if self.cooldown.currentCooldownType ~= COOLDOWN_TYPE_NORMAL then
			self.cooldown:SetEdgeTexture("Interface\\Cooldown\\edge")
			self.cooldown:SetSwipeColor(0, 0, 0)
			self.cooldown:SetHideCountdownNumbers(false)
			self.cooldown.currentCooldownType = COOLDOWN_TYPE_NORMAL
		end

		if locStart > 0 then
			self.cooldown:SetScript("OnCooldownDone", ActionButton_OnCooldownDone );
		end

		if charges and maxCharges and maxCharges > 1 and charges < maxCharges then
			StartChargeCooldown(self, chargeStart, chargeDuration, chargeModRate)
		else
			ClearChargeCooldown(self)
		end

		CooldownFrame_Set(self.cooldown, start, duration, enable, false, modRate)
	end
end

BasicSlot.maxDisplayCount = 9999
function BasicSlot:UpdateCount()
	local show = self:AlwaysShowCount()
		or self:IsConsumableAction()
		or self:IsStackableAction()
		or (not self:IsItemAction() and self:GetActionCount() > 0)

	if show then
		local count = self:GetActionCount()

		if count > self.maxDisplayCount then
			self.Count:SetText("*")
		else
			self.Count:SetText(count)
		end
	else
		local charges, maxCharges = self:GetActionCharges()
		if maxCharges and maxCharges > 1 then
			self.Count:SetText(charges)
		else
			self.Count:SetText("")
		end
	end
end

-- Add a green border if button is an equipped item
function BasicSlot:UpdateEquipped()
	if self:IsEquippedAction() and Addon:ShowingActionButtonEquippedItemBorders() then
		self.Border:Show()
	else
		self.Border:Hide()
	end
end

function BasicSlot:UpdateFlash()
	if (self:IsAttackAction() and self:IsCurrentAction()) or self:IsAutoRepeatAction() then
		self:StartFlashing()
	else
		self:StopFlashing()
	end
end

function BasicSlot:UpdateFlyout()
	-- todo if its necessary to implement custom flyouts
end

function BasicSlot:UpdateIcon()
	local texture = self:GetActionTexture()

	self.zoneAbilityDisabled = false
	self.icon:SetDesaturated(false)

	local type, id = self:GetActionInfo()
	if (type == "spell" or type == "companion") and ZoneAbilityFrame and ZoneAbilityFrame.baseName and not HasZoneAbility() then
		local name = GetSpellInfo(ZoneAbilityFrame.baseName)
		local abilityName = GetSpellInfo(id)

		if name == abilityName then
			texture = GetLastZoneAbilitySpellTexture()
			self.zoneAbilityDisabled = true
			self.icon:SetDesaturated(true)
		end
	end

	-- Update icon and hotkey text
	if texture then
		self.icon:SetTexture(texture)
		self.icon:Show()
	else
		self.icon:Hide()
		self.cooldown:Hide()
	end
end

BasicSlot.hasAction = false
function BasicSlot:UpdateHasAction()
	local hasAction = self:HasAction()

	if hasAction ~= self.hasAction then
		self.hasAction = hasAction

		if hasAction then
			Addon:Notify("OnActionSlotGainedAction", self)
		else
			Addon:Notify("OnActionSlotLostAction", self)
		end
	end
end

function BasicSlot:UpdateHighlightMark() end

function BasicSlot:UpdateOverlayGlow()
	local spellID = self:GetSpellID()

	if spellID ~= nil and IsSpellOverlayed(spellID) then
		self:ShowOverlayGlow()
	else
		self:HideOverlayGlow()
	end
end

BasicSlot.ShowOverlayGlow = ShowOverlayGlow
BasicSlot.HideOverlayGlow = HideOverlayGlow

function BasicSlot:UpdateGrid()
	local alpha

	if self:HasAction() then
		alpha = 1
	elseif Addon:ShowingEmptyActionButtons() then
		alpha = 1
	else
		alpha = self.showGrid > 0 and 1 or 0
	end

	self:SetAlpha(alpha)
end

function BasicSlot:SetLocked(lock)
	if InCombatLockdown() then return end

	self:SetAttribute("lockactionbar", lock)
end

function BasicSlot:UpdatePushed()
	if self:GetButtonState() == 'PUSHED' and not self:GetChecked() then
		self:SetButtonState('NORMAL')
	end
end

function BasicSlot:UpdateSpellHighlightMark() end

function BasicSlot:UpdateState()
	self:SetChecked(self:IsCurrentAction() or self:IsAutoRepeatAction())
end

-- Update Action Text
function BasicSlot:UpdateText()
	local show = self:AlwaysShowLabel() or (
		Addon:ShowingActionButtonLabels()
		and not self:IsConsumableAction()
		and not self:IsStackableAction()
		and (self:IsItemAction() or self:GetActionCount() == 0)
	)

	if show then
		self.Name:SetText(self:GetActionText())
	else
		self.Name:SetText("")
	end
end

function BasicSlot:UpdateTooltip()
	local tooltip = GameTooltip

	if tooltip:IsOwned(self) then
		self:SetTooltip(tooltip)
	end
end

function BasicSlot:UpdateUsable(force)
	if force then
		self.colorId = nil
	end

    local isUsable, notEnoughMana = self:IsUsableAction()

	--usable (ignoring target information)
	if isUsable then
        local inRange = self:IsActionInRange()

		--but out of range
		if inRange == false then
			self:SetColor('oor')
		else
			self:SetColor('normal')
		end
	--out of mana
	elseif notEnoughMana then
		self:SetColor('oom')
	--unusable
	else
		self:SetColor('unusable')
	end
end

function BasicSlot:SetColor(key)
	if self.colorId ~= key then
		self.colorId = key

		local color = SLOT_COLORS[key]
		self.icon:SetVertexColor(color[1], color[2], color[3])
	end
end

-- properties
function BasicSlot:SetAttributes(...)
	if InCombatLockdown() then return end

	local changed = false

	for i = 1, select("#", ...), 2 do
		local key, value = select(i, ...)
		if self:GetAttribute(key) ~= value then
			self:SetAttribute(key, value)
			changed = true
		end
	end

	return changed
end

function BasicSlot:SetKind(kind, action)
	local state = self:GetAttribute("state") or DEFAULT_STATE
	local kindKey = strjoin("--", "kind", state)
	local actionKey = strjoin("--", "kaction", state)

	kind = kind or "empty"
	action = action or ""

	if self:SetAttributes(kindKey, kind, actionKey, action) then
		self:Execute([[ self:RunAttribute("UpdateKind") ]])
		return true
	end
end

function BasicSlot:SetTooltip()
end

function BasicSlot:GetKind()
	local state = self:GetAttribute("state") or DEFAULT_STATE
	local kindKey = strjoin("--", "kind", state)
	local actionkey = strjoin("--", "kaction", state)
	local kind = self:GetAttribute(kindKey)
	local action = self:GetAttribute(actionkey) or ""

	if kind then
		return kind, action
	end

	kindKey = strjoin("--", "kind", DEFAULT_STATE)
	actionkey = strjoin("--", "kaction", DEFAULT_STATE)
	kind = self:GetAttribute(kindKey) or "empty"
	action = self:GetAttribute(actionkey) or ""

	return kind, action
end

BasicSlot.showGrid = 0

function BasicSlot:ShowGrid()
	self.showGrid = self.showGrid + 1
	self:UpdateGrid()
end

function BasicSlot:HideGrid()
	self.showGrid = self.showGrid - 1
	self:UpdateGrid()
end

-- bindings helpers
function BasicSlot:UpdateHotkey()
	local key = self:GetHotkey()

	if key ~= "" and Addon:ShowingActionButtonBindings() then
		self.HotKey:SetText(key)
		self.HotKey:Show()
	else
		self.HotKey:Hide()
	end
end

--returns what hotkey to display for the button
function BasicSlot:GetHotkey()
	local key = (Addon.BindingsManager:GetFrameBindings(self:GetName()))

	if key then
		return LibKeyBound:ToShortKey(key)
	end

	return ""
end

--returns a comma separated list of all bindings for the given action button
--used for keybound support
function BasicSlot:GetBindings()
	return strjoin(", ", Addon.BindingsManager:GetFrameBindings(self:GetName()))
end

--set bindings (more keybound support)
function BasicSlot:SetKey(key)
	Addon.BindingsManager:SetBinding(key, self:GetName())
end

--clears all bindings from the button (keybound support again)
function BasicSlot:ClearBindings()
	Addon.BindingsManager:ClearFrameBindings(self:GetName())
end

--[[ Required Methods ]]--

local function requireMethods(target, ...)
	for i = 1, select("#", ...) do
		local name = select(i, ...)

		target[name] = function(self)
			local msg = ("%q is not defined on ActionSlot kind %q"):format(name, self.kind)

			error(msg, 3)
		end
	end
end

requireMethods(BasicSlot,
	"GetActionCharges",
	"GetActionCooldown",
	"GetActionCount",
	"GetActionID",
	"GetActionInfo",
	"GetActionLossOfControlCooldown",
	"GetActionText",
	"GetActionTexture",
	"GetItemID",
	"GetSpellID",
	"HasAction",
	"IsActionInRange",
	"IsAttackAction",
	"IsAutoCastAction",
	"IsAutoRepeatAction",
	"IsConsumableAction",
	"IsCurrentAction",
	"IsEquippedAction",
	"IsItemAction",
	"IsStackableAction",
	"IsUsableAction"
)

function BasicSlot:HasAction()
	return false
end

function BasicSlot:AlwaysShowLabel()
	return false
end

function BasicSlot:AlwaysShowCount()
	return false
end

Addon.BasicSlot = BasicSlot