local Addon = _G[...]
local FlyoutSlot = Addon.ActionSlots:CreateKind("flyout")

local flyoutSpellBookIndexes = setmetatable({}, {__index = function(t, k)
    local flyoutName = _G.GetFlyoutInfo(k)

	for i = 1, _G.GetNumSpellTabs() do
        local offset, numSpells = select(3, _G.GetSpellTabInfo(i))
		local tabEnd = 1 + offset + numSpells;

        for j = offset, tabEnd - 1 do
			--to get spell info by slot, you have to pass in a pet argument
			local spellType = _G.GetSpellBookItemInfo(j, BOOKTYPE_SPELL);
            if spellType == "FLYOUT" then
                local spellName = _G.GetSpellBookItemName(j, BOOKTYPE_SPELL)

                if spellName == flyoutName then
                    t[k] = j
                    return j
                end
            end
        end
    end
end})

function FlyoutSlot:GetActionCharges()
    return 0, 0, 0, 0
end

function FlyoutSlot:GetActionCooldown()
    return 0, 0, 0, 0
end

function FlyoutSlot:GetActionCount()
    return 0
end

function FlyoutSlot:GetActionID()
    return 0
end

function FlyoutSlot:GetActionInfo()
    return "flyout", self:GetAttribute("spell")
end

function FlyoutSlot:GetActionLossOfControlCooldown()
    return 0, 0
end

function FlyoutSlot:GetActionText()
    return ""
end

function FlyoutSlot:GetActionTexture()
    local index, bookType = self:GetSpellBookIndex()

    if index then
        return GetSpellBookItemTexture(index, bookType)
    end
end

function FlyoutSlot:GetItemID()
    return 0
end

function FlyoutSlot:GetSpellID()
    return 0
end

function FlyoutSlot:HasAction()
    return false
end

function FlyoutSlot:IsActionInRange()
end

function FlyoutSlot:IsAttackAction()
    return false
end

function FlyoutSlot:IsAutoCastAction()
    return false
end

function FlyoutSlot:IsAutoRepeatAction()
    return false
end

function FlyoutSlot:IsConsumableAction()
    return false
end

function FlyoutSlot:IsCurrentAction()
    -- maybe set this if the flyout is currently open
    return false
end

function FlyoutSlot:IsEquippedAction()
    return false
end

function FlyoutSlot:IsItemAction()
    return false
end

function FlyoutSlot:IsStackableAction()
    return false
end

function FlyoutSlot:IsUsableAction()
    return true
end

function FlyoutSlot:GetSpellBookIndex()
    return flyoutSpellBookIndexes[self:GetAttribute("spell")], BOOKTYPE_SPELL
end

function FlyoutSlot:SetTooltip(tooltip)
    local index, bookType = self:GetSpellBookIndex()

    if index then
        tooltip:SetSpellBookItem(index, bookType)
        return
    end
end

function FlyoutSlot.Pickup(flyoutID)
    if flyoutID then
        local index = flyoutSpellBookIndexes[flyoutID]

        if index then
            return _G.PickupSpellBookItem(index, BOOKTYPE_SPELL)
        end
    end
end

Addon.FlyoutSlot = FlyoutSlot