local Addon = _G[...]
local ItemSlot = Addon.ActionSlots:CreateKind("item")

local GetItemIDFromString = Addon.Util.GetItemIDFromString

function ItemSlot:GetActionCharges()
    local spellID = self:GetSpellID()

    if spellID then
        return _G.GetSpellCharges(spellID)
    end

    return 0, 0, 0, 0, 0
end

function ItemSlot:GetActionCooldown()
    return _G.GetItemCooldown(self:GetItemID())
end

function ItemSlot:GetActionCount()
    return _G.GetItemCount(self:GetAttribute("item"))
end

function ItemSlot:GetActionID()
    return 0
end

function ItemSlot:GetActionInfo()
    return "item", self:GetAttribute("item")
end

function ItemSlot:GetActionLossOfControlCooldown()
    local spellID = self:GetSpellID()

    if spellID then
        return GetSpellLossOfControlCooldown(spellID)
    end

    return 0, 0
end

function ItemSlot:GetActionText()
    return ""
end

function ItemSlot:GetActionTexture()
    return _G.GetItemIcon(self:GetItemID())
end

function ItemSlot:GetItemID()
    return GetItemIDFromString(self:GetAttribute("item"))
end

function ItemSlot:GetSpellID()
    local itemID = self:GetItemID()
    if itemID then
        local _, spellID = _G.GetItemSpell(itemID)
        return spellID
    end
end

function ItemSlot:HasAction()
    return self:GetItemID() and true or false
end

function ItemSlot:IsActionInRange()
    local inRange, outOfMana = _G.IsItemInRange(self:GetItemID())

    if inRange ~= nil then
        return inRange, outOfMana
    end

    local spellID = self:GetSpellID()

    if spellID then
        return _G.IsSpellInRange(spellID)
    end
end

function ItemSlot:IsAttackAction()
    local spellID = self:GetSpellID()

    if spellID then
        return _G.IsAttackSpell(spellID)
    end
end

function ItemSlot:IsAutoCastAction()
    local spellID = self:GetSpellID()

    if spellID then
        return GetSpellAutocast(spellID)
    end
end

function ItemSlot:IsAutoRepeatAction()
    local spellID = self:GetSpellID()

    if spellID then
        return _G.IsAutoRepeatSpell(spellID)
    end
end

function ItemSlot:IsConsumableAction()
    return _G.IsConsumableItem(self:GetItemID())
end

function ItemSlot:IsCurrentAction()
    return _G.IsCurrentItem(self:GetItemID())
end

function ItemSlot:IsEquippedAction()
    return _G.IsEquippedItem(self:GetAttribute("item"))
end

function ItemSlot:IsItemAction()
    return true
end

function ItemSlot:IsStackableAction()
    local stackCount = select(8, GetItemInfo(self:GetItemID()))
    return stackCount and stackCount > 1
end

function ItemSlot:IsUsableAction()
    local itemID = self:GetItemID()

    if IsEquippableItem(itemID) then
        return not InCombatLockdown()
    end

    return IsUsableItem(itemID)
end

function ItemSlot:SetTooltip(tooltip)
    local itemID = self:GetItemID()

    if itemID then
        tooltip:SetItemByID(itemID)
    end
end

Addon.ItemSlot = ItemSlot