local Addon = _G[...]
local OverrideActionSlot = Addon.ActionSlots:CreateKind("overrideaction")

-- charges, maxCharges, chargeStart, chargeDuration
function OverrideActionSlot:GetActionCharges()
    return GetActionCharges(self:GetAttribute("action"))
end

function OverrideActionSlot:GetActionCooldown()
    return GetActionCooldown(self:GetAttribute("action"))
end

function OverrideActionSlot:GetActionCount()
    return GetActionCount(self:GetAttribute("action")) or 0
end

function OverrideActionSlot:GetActionID()
    return self:GetAttribute("action")
end

function OverrideActionSlot:GetActionInfo()
    return GetActionInfo(self:GetAttribute("action"))
end

function OverrideActionSlot:GetActionLossOfControlCooldown()
    return GetActionLossOfControlCooldown(self:GetAttribute("action"))
end

function OverrideActionSlot:GetActionText()
    return GetActionText(self:GetAttribute("action"))
end

function OverrideActionSlot:GetActionTexture()
    if self:GetAttribute("action") > 120 then
        return GetActionTexture(self:GetAttribute("action"))
    end

    return [[Interface\ICONS\INV_Misc_EngGizmos_27]]
end

function OverrideActionSlot:GetItemID()
    return 0
end

function OverrideActionSlot:GetSpellID()
    local actionType, id = GetActionInfo(self.action)

    if actionType == "spell" then
        return id
    end
end

function OverrideActionSlot:HasAction()
    return HasAction(self:GetAttribute("action"))
end

function OverrideActionSlot:IsActionInRange()
    return IsActionInRange(self:GetAttribute("action"))
end

function OverrideActionSlot:IsAutoCastAction()
    return false, false
end

function OverrideActionSlot:IsAttackAction()
    return IsAttackAction(self:GetAttribute("action"))
end

function OverrideActionSlot:IsAutoRepeatAction()
    return IsAutoRepeatAction(self:GetAttribute("action"))
end

function OverrideActionSlot:IsConsumableAction()
    return IsConsumableAction(self:GetAttribute("action"))
end

function OverrideActionSlot:IsCurrentAction()
    return IsCurrentAction(self:GetAttribute("action"))
end

function OverrideActionSlot:IsEquippedAction()
    return IsEquippedAction(self:GetAttribute("action"))
end

function OverrideActionSlot:IsItemAction()
    return IsItemAction(self:GetAttribute("action"))
end

function OverrideActionSlot:IsStackableAction()
    return IsStackableAction(self:GetAttribute("action"))
end

function OverrideActionSlot:IsUsableAction()
    if self:GetAttribute("action") > 120 then
        return IsUsableAction(self:GetAttribute("action"))
    end

    return false
end

function OverrideActionSlot:SetTooltip(tooltip)
    local actionID = self:GetAttribute("action")

    if actionID > 120 then
        return tooltip:SetAction(actionID)
    end

    local name = ("Override Action %d"):format(actionID)
    GameTooltip_SetTitle(tooltip, name, HIGHLIGHT_FONT_COLOR)
end

function OverrideActionSlot.Pickup(action)
    return Addon.SecureEnv:PickupCustom(
        "overrideaction",
        action,
        [[Interface\ICONS\INV_Misc_EngGizmos_27]]
    )
end


Addon.OverrideActionSlot = OverrideActionSlot