local Addon = _G[...]
local PetAbilitySlot = Addon.ActionSlots:CreateKind("petability")
local PetActions = Addon.PetActions
local IsAttackSpellID = Addon.Util.IsAttackSpellID
local IsAutoRepeatSpellID = Addon.Util.IsAutoRepeatSpellID
local IsSpellIDInRange = Addon.Util.IsSpellIDInRange

local function withSpellID(with, without)
    if type(without) == "function" then
        return function(self)
            local spellID = self:GetSpellID()

            if spellID then
                return with(spellID)
            end

            return without(self)
        end
    end

    return function(self)
        local spellID = self:GetSpellID()

        if spellID then
            return with(spellID)
        end
    end
end

-- charges, maxCharges, chargeStart, chargeDuration
PetAbilitySlot.GetActionCharges = withSpellID(
    GetSpellCharges,
    function()
        return 0, 0, 0, 0, 0
    end
)

PetAbilitySlot.GetActionCooldown = withSpellID(
    GetSpellCooldown,
    function()
        return 0, 0, 0, 0
    end
)

PetAbilitySlot.GetActionCount = withSpellID(
    GetSpellCount,
    function()
        return 0
    end
)

function PetAbilitySlot:GetActionID()
    return 0
end

function PetAbilitySlot:GetActionInfo()
    return "spell", self:GetSpellID(), "pet"
end

PetAbilitySlot.GetActionLossOfControlCooldown = withSpellID(
    GetSpellLossOfControlCooldown,
    function()
        return 0, 0
    end
)

function PetAbilitySlot:GetActionText()
    return ""
end

function PetAbilitySlot:GetActionTexture()
    local actionID = self:GetAttribute("petActionID")

    if actionID then
        return (select(3, PetActions:GetPetActionIDInfo(actionID)))
    end
end

function PetAbilitySlot:GetItemID()
    return 0
end

function PetAbilitySlot:GetSpellID()
    local actionID = self:GetAttribute("petActionID")

    if actionID then
        return (select(4, PetActions:GetPetActionIDInfo(actionID)))
    end
end

function PetAbilitySlot:HasAction()
    return true
end

PetAbilitySlot.IsActionInRange = withSpellID(IsSpellIDInRange)

PetAbilitySlot.IsAttackAction = withSpellID(IsAttackSpellID)

PetAbilitySlot.IsAutoCastAction = withSpellID(GetSpellAutocast)

PetAbilitySlot.IsAutoRepeatAction = withSpellID(IsAutoRepeatSpellID)

PetAbilitySlot.IsConsumableAction = withSpellID(IsConsumableSpell)

PetAbilitySlot.IsCurrentAction = withSpellID(IsCurrentSpell)

function PetAbilitySlot:IsEquippedAction()
    return false
end

function PetAbilitySlot:IsItemAction()
    return false
end

function PetAbilitySlot:IsStackableAction()
    return false
end

PetAbilitySlot.IsUsableAction = withSpellID(IsUsableSpell)

function PetAbilitySlot:SetTooltip(tooltip)
    local spellID = self:GetAttribute("spell")

    if spellID then
        return tooltip:SetSpellByID(self:GetSpellID())
    end
end

Addon.PetAbilitySlot = PetAbilitySlot