-- petActions.lua
-- an API for grabbing info via pet actionIDs

local Addon = _G[...]

local GetActionInfo = _G.GetActionInfo
local GetActionTexture = _G.GetActionTexture
local GetSpellBookItemInfo = _G.GetSpellBookItemInfo
local GetSpellBookItemTexture = _G.GetSpellBookItemTexture
local GetSpellBookItemName = _G.GetSpellBookItemName
local GetSpellInfo = _G.GetSpellInfo
local Memoize = Addon.Util.Memoize

local PET_COMMANDS = {
    [117440512] = SLASH_PET_STAY1,
    [117440513] = SLASH_PET_FOLLOW1,
    [117440514] = SLASH_PET_ATTACK1,
    [117440516] = SLASH_PET_MOVE_TO1,
    [100663296] = SLASH_PET_PASSIVE1,
    [100663297] = SLASH_PET_DEFENSIVE1,
    [100663298] = SLASH_PET_AGGRESSIVE1, -- this is a guess
    [100663299] = SLASH_PET_ASSIST1,
}

-- this one is kindof a dumb one, but it prevents the need to look up
-- the name of action slots, which would probably require tooltip parsing
local textureIDToPetActionID = Memoize(function(id)
    local i = 1
    local spellType, petActionID, textureID

    repeat
        spellType, petActionID = GetSpellBookItemInfo(i, BOOKTYPE_PET)
        textureID = GetSpellBookItemTexture(i, BOOKTYPE_PET)

        if textureID == id then
            return petActionID
        end

        i = i + 1
    until not spellType
end)

local spellIDToPetActionID = Memoize(function(id)
    local i = 1
    local spellType, petActionID, spellID

    repeat
        spellType, petActionID = GetSpellBookItemInfo(i, BOOKTYPE_PET)
        spellID = select(7, GetSpellInfo(i, BOOKTYPE_PET))

        if spellID == id then
            return petActionID
        end

        i = i + 1
    until not spellType
end)

local nameToPetActionID = Memoize(function(name)
    local i = 1
    local spellType, petActionID

    repeat
        spellType, petActionID = GetSpellBookItemInfo(i, BOOKTYPE_PET)

        if GetSpellBookItemName(i, BOOKTYPE_PET) == name then
            return petActionID
        end

        i = i + 1
    until not spellType
end)

local petActionIDInfo = Memoize(function(actionID)
    local i = 1
    local spellType, id

    repeat
        spellType, id = GetSpellBookItemInfo(i, BOOKTYPE_PET)

        if actionID == id then
            local name, kind, spellID = GetSpellBookItemName(i, BOOKTYPE_PET)
            local texture = GetSpellBookItemTexture(i, BOOKTYPE_PET)
            local command = PET_COMMANDS[actionID]

            kind = kind or ""

            if command then
                return {name, kind, texture, false, PET_COMMANDS[actionID]}
            end

            return {name, kind, texture, spellID}
        end

        i = i + 1
    until not spellType
end)

local PetActions = {}

function PetActions:GetPetActionIDFromActionSlot(slot)
    local type, spellID, bookType = GetActionInfo(slot)

    if type == "spell" and bookType == BOOKTYPE_PET then
        if spellID ~= 0 then
            return spellIDToPetActionID[spellID]
        end

        return textureIDToPetActionID[GetActionTexture(slot)]
    end
end

function PetActions:GetPetActionIDFromPetActionSlot(slot)
    local name, _, isToken, _, _, _, spellID = GetPetActionInfo(slot)

    if isToken == 1 then
        name = _G[name]
        return nameToPetActionID[name]
    end

    if spellID then
        return spellIDToPetActionID[spellID]
    end
end

-- return name, kind, texture, spellID or false, command
function PetActions:GetPetActionIDInfo(actionID)
    local info = petActionIDInfo[actionID]

    if info then
        return unpack(info)
    end
end

function PetActions:IsPetAttackAction(actionID)
    return actionID == 117440514
end

function PetActions.Pickup(actionID)
    local i = 1
    local spellType, petActionID

    repeat
        spellType, petActionID = GetSpellBookItemInfo(i, BOOKTYPE_PET)

        if petActionID == actionID then
            return PickupSpellBookItem(i, BOOKTYPE_PET)
        end

        i = i + 1
    until not spellType
end

Addon.PetActions = PetActions