local Addon = _G[...]
local SpellSlot = Addon.ActionSlots:CreateKind("spell")
local IsAttackSpellID = Addon.Util.IsAttackSpellID
local IsAutoRepeatSpellID = Addon.Util.IsAutoRepeatSpellID
local IsSpellIDInRange = Addon.Util.IsSpellIDInRange

-- charges, maxCharges, chargeStart, chargeDuration
function SpellSlot:GetActionCharges()
    return _G.GetSpellCharges(self:GetAttribute("spell"))
end

function SpellSlot:GetActionCooldown()
    return _G.GetSpellCooldown(self:GetAttribute("spell"))
end

function SpellSlot:GetActionCount()
    return 0
end

function SpellSlot:GetActionID()
    return 0
end

function SpellSlot:GetActionInfo()
    return "spell", self:GetAttribute("spell"), "spell"
end

function SpellSlot:GetActionLossOfControlCooldown()
    return GetSpellLossOfControlCooldown(self:GetAttribute("spell"))
end

function SpellSlot:GetActionText()
    return ""
end

function SpellSlot:GetActionTexture()
    return (select(3, _G.GetSpellInfo(self:GetAttribute("spell"))))
end

function SpellSlot:GetItemID()
    return 0
end

function SpellSlot:GetSpellID()
    return self:GetAttribute("spell")
end

function SpellSlot:HasAction()
    return true
end

function SpellSlot:IsActionInRange()
    local spellID = self:GetAttribute("spell")

    if spellID then
        return IsSpellIDInRange(spellID)
    end
end

function SpellSlot:IsAttackAction()
    local spellID = self:GetAttribute("spell")

    if spellID then
        return IsAttackSpellID(spellID)
    end
end

function SpellSlot:IsAutoCastAction()
    local spellID = self:GetAttribute("spell")

    if spellID then
        return GetSpellAutocast(spellID)
    end
end

function SpellSlot:IsAutoRepeatAction()
    local spellID = self:GetAttribute("spell")

    if spellID then
        return IsAutoRepeatSpellID(spellID)
    end
end

function SpellSlot:IsConsumableAction()
    return _G.IsConsumableSpell(self:GetAttribute("spell")) == 1
end

function SpellSlot:IsCurrentAction()
    local spellID = self:GetAttribute("spell")

    if spellID then
        return _G.IsCurrentSpell(spellID)
    end
end

function SpellSlot:IsEquippedAction()
    return false
end

function SpellSlot:IsItemAction()
    return false
end

function SpellSlot:IsStackableAction()
    return false
end

function SpellSlot:IsUsableAction()
    local spellID = self:GetAttribute("spell")

    if spellID then
        return _G.IsUsableSpell(spellID)
    end
end

function SpellSlot:SetTooltip(tooltip)
    local spellID = self:GetAttribute("spell")

    if spellID then
        return tooltip:SetSpellByID(self:GetSpellID())
    end
end

Addon.SpellSlot = SpellSlot