
-- VirtualButton, a button for clicking another button
-- this is used to allow for using action slots on key press without
-- actually needing to register that button for press on keyDown
local AddonName = ...
local Addon = _G[...]
local VirtualButton = {}

local nextName
do
	local id = 0

	nextName = function()
		id = id + 1
		return ('%sVirtualButton%d'):format(AddonName, id)
	end
end

function VirtualButton:Create(target)
	local button = CreateFrame('Button', nextName(), nil, 'SecureActionButtonTemplate')

	button:Hide()
	button:SetAttribute('type', 'click')
	button:SetAttribute('owner', target:GetName())
	button:SetAttribute('clickbutton', target)
	button:SetScript('OnMouseUp', self.OnMouseUp)
	button:SetScript('OnMouseDown', self.OnMouseDown)

	if GetCVarBool('ActionButtonUseKeyDown') then
		button:RegisterForClicks('anyDown')
	else
		button:RegisterForClicks('anyUp')
	end

	return button
end

function VirtualButton:OnMouseUp()
	local target = self:GetAttribute('clickbutton')

	if target then
		target:SetButtonState('NORMAL')
	end
end

function VirtualButton:OnMouseDown()
	local target = self:GetAttribute('clickbutton')

	if target then
		target:SetButtonState('PUSHED')
	end
end

Addon.VirtualButton = VirtualButton