local AddonName = ...
local Addon = _G[AddonName]

local DB_NAME = AddonName .. "DB"
local DB_VERSION = 1
local DISABLED_PAGE = -1

function Addon:OnProfileChanged(...)
    self:OnProfileLoaded()
	self:Notify("OnProfileLoaded")
end

function Addon:OnProfileCreated(...)
    self:OnProfileLoaded()
	self:Notify("OnProfileLoaded")
end

function Addon:OnProfileCopied(...)
    self:OnProfileLoaded()
	self:Notify("OnProfileLoaded")
end

function Addon:OnProfileDeleted(...)
end

function Addon:OnProfileReset(...)
    self:OnProfileLoaded()
	self:Notify("OnProfileLoaded")
end

function Addon:GetDatabaseDefaults()
    local defaults = {
        global = {
			stickyBars = false,

            linkedBars = false,

			launcher = {
				hide = false
            },

            numGamepadPages = 10,
        },

        profile = {
            castOnKeyPress = GetCVarBool("ActionButtonUseKeyDown"),

			showEmptyActionButtons = true,
			showActionButtonBindings = true,
			showActionButtonLabels = true,
			showActionButtonEquippedItems = true,
			showActionButtonTooltips = true,
            showActionButtonTooltipsInCombat = true,

            -- gamePad layout settings
            deviceMode = Addon.DeviceModes.Automatic,
            deviceType = Addon.DeviceTypes.Orbweaver,
            scale = 1,
            spacing = 10,
            padW = 6,
            padH = 6,
            regionSpacing = 16,
            shown = true,
            point = "CENTER",
            x = 0,
            y = 0,
            useAsOverrideBar = false,

            regionStateMaps = {
                -- RegionId
                ["*"] = {
                    -- [StateId] = PageNumber
                    ["**"] = DISABLED_PAGE
                },

                [1] = {
                    ["page2"] = 2,
                    ["page3"] = 3,
                    ["page4"] = 4,
                    ["page5"] = 5,
                    ["page6"] = 6,
                    ["shadowdance"] = 7,
                    ["stealth"] = 7,
                    ["cat"] = 7,
                    ["bear"] = 8,
                    ["moonkin"] = 9,
                }
            },

            bindings = {
                -- ActionSlotId = {keys}
                [AddonName .. "ActionButton101"] = "F6",
                [AddonName .. "ActionButton102"] = "F7",
                [AddonName .. "ActionButton103"] = "F8",
                [AddonName .. "ActionButton104"] = "F9",
                [AddonName .. "ActionButton105"] = "F10",
                [AddonName .. "ActionButton106"] = "F11",
                [AddonName .. "ActionButton107"] = "Q",
                [AddonName .. "ActionButton108"] = "W",
                [AddonName .. "ActionButton109"] = "E",
                [AddonName .. "ActionButton110"] = "F12",
                [AddonName .. "ActionButton111"] = "DELETE",
                [AddonName .. "ActionButton112"] = "A",
                [AddonName .. "ActionButton113"] = "S",
                [AddonName .. "ActionButton114"] = "D",
                [AddonName .. "ActionButton115"] = "F",
                [AddonName .. "ActionButton116"] = "INSERT",
                [AddonName .. "ActionButton117"] = ".",
                [AddonName .. "ActionButton118"] = "'",
                [AddonName .. "ActionButton119"] = ";",
                [AddonName .. "ActionButton120"] = ",",
                [AddonName .. "ActionButton201"] = "PAGEUP",
                [AddonName .. "ActionButton203"] = "[",
                [AddonName .. "ActionButton205"] = "PAGEDOWN",
                [AddonName .. "ActionButton207"] = "]",
                [AddonName .. "ActionButton209"] = "`",
                [AddonName .. "ActionButton210"] = "SPACE"
            },

            actionSlotContents = {
                -- CharKey
                ["*"] = {
                    -- SpecGroup
                    ["*"] = {
                        [101] = {
                            override = {
                                kind = "overrideaction",
                                kaction = 1
                            }
                        },
                        [102] = {
                            override = {
                                kind = "overrideaction",
                                kaction = 2
                            }
                        },
                        [103] = {
                            override = {
                                kind = "overrideaction",
                                kaction = 3
                            }
                        },
                        [104] = {
                            override = {
                                kind = "overrideaction",
                                kaction = 4
                            }
                        },
                        [105] = {
                            override = {
                                kind = "overrideaction",
                                kaction = 5
                            }
                        },
                        [106] = {
                            override = {
                                kind = "overrideaction",
                                kaction = 6
                            }
                        },
                        [107] = {
                            default = {
                                kind = "movement",
                                kaction = "TURNLEFT"
                            }
                        },
                        [108] = {
                            default = {
                                kind = "movement",
                                kaction = "MOVEFORWARD"
                            }
                        },
                        [109] = {
                            default = {
                                kind = "movement",
                                kaction = "TURNRIGHT"
                            }
                        },
                        [110] = {
                            override = {
                                kind = "exitvehicle",
                                kaction = ""
                            }
                        },
                        [112] = {
                            default = {
                                kind = "movement",
                                kaction = "STRAFELEFT"
                            }
                        },
                        [113] = {
                            default = {
                                kind = "movement",
                                kaction = "MOVEBACKWARD"
                            }
                        },
                        [114] = {
                            default = {
                                kind = "movement",
                                kaction = "STRAFERIGHT"
                            }
                        },

                        [201] = {
                            default = {
                                kind = "actionpage",
                                kaction = 4
                            }
                        },

                        [203] = {
                            default = {
                                kind = "actionpage",
                                kaction = 1
                            }
                        },

                        [205] = {
                            default = {
                                kind = "actionpage",
                                kaction = 2
                            }
                        },

                        [207] = {
                            default = {
                                kind = "actionpage",
                                kaction = 3
                            }
                        },

                        [210] = {
                            default = {
                                kind = "movement",
                                kaction = "JUMP"
                            }
                        }
                    }
                }
            }
        }
    }

    -- apply defaults for all known buttonIDs
    local actionSlotContents = defaults.profile.actionSlotContents["*"]["*"]

    for _, layout in pairs(Addon.DeviceLayouts) do
        for _, region in pairs(layout) do
            for _, id in pairs(region.buttons) do
                if not actionSlotContents[id] then
                    actionSlotContents[id] = {
                        default = {
                            kind = "empty",
                            kaction = ""
                        }
                    }
                elseif not actionSlotContents[id].default then
                    actionSlotContents[id].default = {
                        kind = "empty",
                        kaction = ""
                    }
                end
            end
        end
    end

    return defaults
end

function Addon:InitializeDatabase()
	local existingDB = _G[DB_NAME]
	if existingDB and not (existingDB.global and existingDB.global.version == DB_VERSION) then
		self:RunMigrations(existingDB)
	end

	local db = LibStub("AceDB-3.0"):New(DB_NAME, self:GetDatabaseDefaults())

	db.global.version = DB_VERSION
	db.RegisterCallback(self, "OnNewProfile", "OnProfileCreated")
	db.RegisterCallback(self, "OnProfileChanged")
	db.RegisterCallback(self, "OnProfileCopied")
	db.RegisterCallback(self, "OnProfileReset")
	db.RegisterCallback(self, "OnProfileDeleted")

	self.db = db
end

function Addon:RunMigrations(db)
end

-- profiles
function Addon:SaveProfile(name)
	local toCopy = self.db:GetCurrentProfile()

	if name and name ~= toCopy then
		self.db:SetProfile(name)
	end
end

function Addon:SetProfile(name)
	local profile = self:MatchProfile(name)

	if profile and profile ~= self.db:GetCurrentProfile() then
		self.db:SetProfile(profile)
	end
end

function Addon:DeleteProfile(name)
	local profile = self:MatchProfile(name)

	if profile and profile ~= self.db:GetCurrentProfile() then
		self.db:DeleteProfile(profile)
	end
end

function Addon:CopyProfile(name)
	if name and name ~= self.db:GetCurrentProfile() then
		self.db:CopyProfile(name)
	end
end

function Addon:ResetProfile()
	self.db:ResetProfile()
end

function Addon:MatchProfile(name)
	if not name then return end

	name = tostring(name):lower()
	local nameRealm = strjoin(" - ", name, GetRealmName():lower())
	local match

	for _, k in ipairs(self.db:GetProfiles()) do
		local key = k:lower()
		if key == name then
			return k
		elseif key == nameRealm then
			match = k
		end
	end

    return match
end