﻿-- a panel for configuring global addon settings
local AddonName, Addon = ...
local ParentAddonName = GetAddOnDependencies(AddonName)
local ParentAddon = _G[ParentAddonName]
local L = LibStub("AceLocale-3.0"):GetLocale(ParentAddonName)

local GeneralPanel = Addon.OptionsUI:NewPanel(L.General)

local GeneralSettingsContainer
do
	local frame = CreateFrame('Frame', nil, GeneralPanel)

	local bindButton = GeneralPanel:Add('Button', {
		name = L.EnterBindingMode,
		width = 136,
		height = 22,
		click = function()
			ParentAddon:ToggleBindingMode()
		end
	})
	bindButton:SetPoint('TOPLEFT', frame, 'TOPLEFT', 0, -2)

	--[[ General Settings ]]--

	local lockButton = GeneralPanel:Add('CheckButton', {
		name = L.LockBarPosition,
		get = function() return not ParentAddon:InConfigurationMode() end,
		set = function(_, enable) ParentAddon:SetConfigurationMode(not enable) end
	})
	lockButton:SetPoint('TOPLEFT', bindButton, 'BOTTOMLEFT', 0, -10)

	lockButton.OnConfigurationModeChanged = function(self, msg, enable)
		self:SetChecked(not enable)
	end

	ParentAddon.RegisterCallback(lockButton, "OnConfigurationModeChanged")

	local showLauncherToggle = GeneralPanel:Add('CheckButton', {
		name = L.ShowMinimapButton,
		get = function() return ParentAddon:ShowingLauncher() end,
		set = function(_, enable) ParentAddon:SetShowLauncher(enable) end
	})
	showLauncherToggle:SetPoint('TOPLEFT', lockButton, 'BOTTOMLEFT', 0, -10)

	--[[ Action Bar Settings ]]--

	--show empty buttons
	local showEmptyButtonsToggle = GeneralPanel:Add('CheckButton', {
		name = L.ShowEmptyButtons,
		get = function() return ParentAddon:ShowingEmptyActionButtons() end,
		set = function(_, enable) ParentAddon:SetShowEmptyActionButtons(enable) end
	})
	showEmptyButtonsToggle:SetPoint('TOP', showLauncherToggle, 'BOTTOM', 0, -10)

	--show keybinding text
	local showBindingsButtonToggle = GeneralPanel:Add('CheckButton', {
		name = L.ShowBindingText,
		get = function() return ParentAddon:ShowingActionButtonBindings() end,
		set = function(_, enable) ParentAddon:SetShowActionButtonBindings(enable) end
	})
	showBindingsButtonToggle:SetPoint('TOP', showEmptyButtonsToggle, 'BOTTOM', 0, -10)

	--show macro text
	local showMacroTextToggle = GeneralPanel:Add('CheckButton', {
		name = L.ShowMacroText,
		get = function() return ParentAddon:ShowingActionButtonLabels() end,
		set = function(_, enable) ParentAddon:SetShowActionButtonLabels(enable) end
	})
	showMacroTextToggle:SetPoint('TOP', showBindingsButtonToggle, 'BOTTOM', 0, -10)

	--show equipped i tem borders
	local showEquippedToggle = GeneralPanel:Add('CheckButton', {
		name = L.ShowEquippedItemBorders,
		get = function() return ParentAddon:ShowingActionButtonEquippedItemBorders() end,
		set = function(_, enable) ParentAddon:SetShowActionButtonEquippedItemBorders(enable) end
	})
	showEquippedToggle:SetPoint('TOP', showMacroTextToggle, 'BOTTOM', 0, -10)

	-- show tooltips
	local showTooltipsToggle = GeneralPanel:Add('CheckButton', {
		name = L.ShowTooltips,
		get = function() return ParentAddon:ShowingActionButtonTooltips() end,
		set = function(_, enable) ParentAddon:SetShowActionButtonTooltips(enable) end
	})
	showTooltipsToggle:SetPoint('TOP', showEquippedToggle, 'BOTTOM', 0, -10)

	--show tooltips in combat
	local showTooltipsInCombatToggle = GeneralPanel:Add('CheckButton', {
		name = L.ShowTooltipsCombat,
		small = true,
		get = function() return ParentAddon:ShowingActionButtonTooltipsInCombat() end,
		set = function(_, enable) ParentAddon:SetShowActionButtonTooltipsInCombat(enable) end
	})
	showTooltipsInCombatToggle:SetPoint('TOP', showTooltipsToggle, 'BOTTOM', 8, -2)

	GeneralSettingsContainer = frame
end

GeneralSettingsContainer:SetPoint('TOPLEFT')
GeneralSettingsContainer:SetPoint('TOPRIGHT', GeneralSettingsContainer:GetParent(), 'TOP')
GeneralSettingsContainer:SetPoint('BOTTOMLEFT')