local AddonName, Addon = ...
local ParentAddonName = GetAddOnDependencies(AddonName)
local ParentAddon = _G[ParentAddonName]
local L = LibStub("AceLocale-3.0"):GetLocale(ParentAddonName)

local OptionsUIPanel = Addon:CreateClass('Frame')
do
	function OptionsUIPanel:New(owner, parent)
		local frame = self:Bind(CreateFrame('Frame', nil, parent or owner))

		frame:SetAllPoints(frame:GetParent())
		frame:Hide()

		return frame
	end

	function OptionsUIPanel:Add(widget, options)
		options.parent = self

		return Addon[widget]:New(options)
	end
end

local OptionsUI = {}
do
	OptionsUI.insets = 16

	function OptionsUI:Init()
		self.panels = {}

		local title, description = select(2, GetAddOnInfo(ParentAddonName))

		-- create interface options frame area
		local frame = CreateFrame("Frame", ParentAddonName .. "OptionsUI", _G.UIParent)
		frame:SetScript('OnShow', function() self:OnShow() end)
		frame:Hide()
		frame.name = title
		self.frame = frame

		-- add frame title
		local titleText = frame:CreateFontString(nil, 'ARTWORK', 'GameFontNormalLarge')
		titleText:SetPoint('TOPLEFT', self.insets, -self.insets)
		titleText:SetText(title)

		-- add frame description
		local descriptionText = frame:CreateFontString(nil, 'ARTWORK', 'GameFontHighlightSmall')
		descriptionText:SetPoint('TOPLEFT', titleText, 'BOTTOMLEFT', 0, -8)
		descriptionText:SetText(description)

		-- panel selector
		local panelSelector = Addon.PanelSelector:New(self.frame)
		panelSelector:SetPoint('TOPLEFT', descriptionText, 'BOTTOMLEFT', 0, -4)
		panelSelector:SetPoint('RIGHT', -self.insets, 0)
		panelSelector:SetHeight(20)
		panelSelector.OnSelect = function(_, id) self:ShowPanel(id) end
		self.panelSelector = panelSelector

		-- panel container
		local panelContainer = CreateFrame('Frame', nil, self.frame)
		panelContainer:SetPoint('TOPLEFT', panelSelector, 'BOTTOMLEFT', 0, -4)
		panelContainer:SetPoint('BOTTOMRIGHT', -self.insets, self.insets)
		self.panelContainer = panelContainer

		ParentAddon.RegisterCallback(self, "OnShowOptionsMenu")
		InterfaceOptions_AddCategory(self.frame)
	end


	function OptionsUI:OnShowOptionsMenu()
		InterfaceOptionsFrame_Show()
		InterfaceOptionsFrame_OpenToCategory(self.frame)
	end

	function OptionsUI:OnShow()
		self.panelSelector:Select(L.General)
	end

	function OptionsUI:NewPanel(id)
		self.panelSelector:AddPanel(id)

		local panel = OptionsUIPanel:New(self, self.panelContainer)
		self.panels[id] = panel

		return panel
	end

	function OptionsUI:ShowPanel(id)
		if self.panels then
			for i, panel in pairs(self.panels) do
				if i == id then
					panel:Show()
				else
					panel:Hide()
				end
			end
		end
	end
end

OptionsUI:Init()
Addon.OptionsUI = OptionsUI
